/** @module player/state */
import { call, put, takeEvery } from 'redux-saga/effects'
import { generateCalendar } from './functions'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  fileUploadStart: file => {
    return {
      type: 'CALENDAR_FILE_UPLOAD_START',
      file
    }
  },
  fileUploadSuccess: allMatches => {
    return {
      type: 'CALENDAR_FILE_UPLOAD_SUCCESS',
      allMatches
    }
  },
  fileUploadFailure: error => {
    return {
      type: 'CALENDAR_FILE_UPLOAD_FAILURE',
      alert: { type: 'warning', text: error.toString() }
    }
  },
  filterSortMatches: () => {
    return {
      type: 'CALENDAR_FILTER_SORT'
    }
  }
}
console.log('State actions', actions)

/** state definition */
export const state = {
  allMatches: [],
  filteredMatches: [],
  filters: [
    {
      filterFunction: (match, filterValue) => {
        return match.Konkurrenz === filterValue
      },
      filterValue: 'MS R6/R7'
    }
  ],
  sorting: [
    (match1, match2) => {
      if (match1.Datum > match2.Datum) {
        return 1
      }
      if (match1.Datum === match2.Datum) {
        return 0
      }
      return -1
    },
    (match1, match2) => {
      if (match1.Ort > match2.Ort) {
        return 1
      }
      if (match1.Ort === match2.Ort) {
        return 0
      }
      return -1
    }
  ],
  fileUpload: 'idle',
  file: null
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'CALENDAR_FILE_UPLOAD_START':
      return { ...state, fileUpload: 'started', file: action.file }
    case 'CALENDAR_FILE_UPLOAD_SUCCESS':
      return { ...state, fileUpload: 'finished', allMatches: action.allMatches }
    case 'CALENDAR_FILE_UPLOAD_FAILURE':
      return { ...state, fileUpload: 'failure' }
    case 'CALENDAR_FILTER_SORT':
      const { allMatches, filters, sorting } = state
      const sortedMatches = allMatches.sort((match1, match2) => {
        return sorting.length ? sorting.reduce((acc, sortFun) => {
          return acc === 0 ? sortFun(match1, match2) : acc
        }, 0) : 0
      })
      console.log('Sorted matches', sortedMatches)
      const filteredMatches = sortedMatches.filter(match => {
        return filters.length ? filters.map(filter => filter.filterFunction(match, filter.filterValue)).every(value => !!value) : true
      })
      console.log('Filtered matches', filteredMatches)
      return { ...state, filteredMatches }
    default:
      return state
  }
}

function * uploadFile (action) {
  try {
    console.log('Calendar uploadFile', action.file)
    const allMatches = yield call(generateCalendar, action.file)
    console.log('Calendar success!', actions.fileUploadSuccess(allMatches))
    yield put(actions.fileUploadSuccess(allMatches))
    yield put(actions.filterSortMatches())
  } catch (error) {
    console.log('Calendar failure!', actions.fileUploadFailure(error))
    yield put(actions.fileUploadFailure(error))
  }
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Calendar saga started.')
  yield takeEvery('CALENDAR_FILE_UPLOAD_START', uploadFile)
}
